#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@File    ：test_queue.py
@Author  ：KING
@Date    ：2024/6/11 19:21 
"""
import unittest
from hard_connect.utils import DequeWithMaxLen


class TestDequeWithMaxLen(unittest.TestCase):

    def test_init(self):
        # 测试初始化时设置最大长度
        dq = DequeWithMaxLen(max_length=10)
        self.assertEqual(dq.max_length, 10)

    def test_put(self):
        # 测试put方法，包括队列满时移除最旧元素
        dq = DequeWithMaxLen(max_length=3)
        dq.put(1)
        dq.put(2)
        self.assertEqual(list(dq), [1, 2])

        dq.put(3)
        self.assertEqual(list(dq), [1, 2, 3])

        dq.put(4)
        self.assertEqual(list(dq), [2, 3, 4])  # 1被移除

    def test_popleft(self):
        # 测试popleft方法
        dq = DequeWithMaxLen(max_length=5)
        dq.extend([1, 2, 3])
        self.assertEqual(dq.popleft(), 1)
        self.assertEqual(list(dq), [2, 3])

    def test_get(self):
        # 测试get方法，它是popleft的别名
        dq = DequeWithMaxLen(max_length=5)
        dq.extend([1, 2, 3])
        self.assertEqual(dq.get(), 1)

    def test_get_new_value(self):
        # 测试get_new_value方法
        dq = DequeWithMaxLen(max_length=5)
        dq.extend([1, 2, 3])
        self.assertEqual(dq.get_new_value(), 3)
        self.assertEqual(dq.get_new_value(0), 1)  # 测试索引
        self.assertEqual(len(dq), 3)  # 确保元素未被移除

    def test_clear(self):
        # 测试clear方法
        dq = DequeWithMaxLen(max_length=5)
        dq.extend([1, 2, 3])
        dq.clear()
        self.assertEqual(len(dq), 0)

    def test_count(self):
        # 测试count方法
        dq = DequeWithMaxLen(max_length=5)
        dq.extend([1, 2, 2, 3])
        self.assertEqual(dq.count(2), 2)

    def test_extend(self):
        # 测试extend方法
        dq = DequeWithMaxLen(max_length=5)
        dq.extend([1, 2])
        dq.extend([3, 4])
        self.assertEqual(list(dq), [1, 2, 3, 4])

    def test_max_length_behavior(self):
        # 测试最大长度的行为
        dq = DequeWithMaxLen(max_length=2)
        dq.extend([1, 2, 3])
        self.assertEqual(list(dq), [2, 3])  # 1被移除


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@File    ：dequeue.py
@Author  ：KING
@Date    ：2024/6/11 12:49 
"""
import logging
import logging.config
from typing import Union, Generator
from collections import deque
from abc import ABC, abstractmethod


class BaseConn(ABC):

    def __init__(
            self,
            send_lf='\r\n',
            receive_lf='\r\n',
            queue_max_length=500,
            timeout: int = 1,
            end_of_msg='🔚',
            keep_line_feed=False,
            logging_level=logging.INFO,
            logging_filename=None,
            receive_generator: Generator = None,
    ):
        """
        Base class for communication, including serial, socket, etc.
        :param send_lf:            # send line feed, default: '\r\n'
        :param receive_lf:         # receive line feed, default: '\r\n'
        :param queue_max_length:   # queue max length, default: 500
        :param timeout:            # Send and receive timeout, default: 1s
        :param end_of_msg:         #  special tag data, Set the tag end_of_msg, which is not stored in the queue. Default  🔚
        :param keep_line_feed:     # Write queue data to preserve line breaks  default: False
        :param logging_level:      # Logging level, default: logging.INFO, DEBUG: write file and console, INFo: write console  # noqa
        :param logging_filename:   # Logging file name, default: None, if set, write log to file. If not set use default log setting
        :param receive_generator:  # Generator for receiving data, default: None, if set, use this generator to receive data
        """
        self.conn = None
        self.send_lf = send_lf
        self.receive_lf = receive_lf
        self.end_of_msg = end_of_msg
        self.keep_line_feed = keep_line_feed
        self.timeout = timeout
        self.queue = DequeWithMaxLen(queue_max_length)
        self.logging_filename = logging_filename
        self.receive_generator = receive_generator
        self.init_logging()
        self.logger = logging.getLogger("hard_conn")
        self.logger.setLevel(logging_level)
        super().__init__()

    def __del__(self):
        """
        close socket connection when object is destroyed
        :return:
        """
        self.receive_generator and self.receive_generator.close()
        self.disconnect()

    def __enter__(self):
        not self.conn and self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.receive_generator and self.receive_generator.close()
        self.disconnect()

    def init_logging(self):
        """
        init logging
        :return:
        """
        from hard_connect.log import DEFAULT_LOGGING
        if self.logging_filename:
            DEFAULT_LOGGING['handlers']['file']['filename'] = self.logging_filename
        logging.config.dictConfig(DEFAULT_LOGGING)

    @abstractmethod
    def connect(self):
        """
        connect to the server
        :return:
        """
        pass

    @abstractmethod
    def disconnect(self):
        """
        disconnect from the server
        :return:
        """
        pass

    @abstractmethod
    def send(self, send_str: Union[str, bytes]):
        """
        send message to the server
        :param send_str:
        :return:
        """
        pass

    @abstractmethod
    def receive(self):
        """
        receive message from the server
        :return:
        """
        pass

    def put_queue(self, bytes_recv_line: bytes = None, bytes_recv_multi_lines: list = None):
        """
        sub thread receive data, put data to queue

        :param bytes_recv_line:
        :param bytes_recv_multi_lines:
        :return:
        """
        try:
            if bytes_recv_line is not None:
                bytes_recv_multi_lines = [bytes_recv_line]

            lines = [line.decode() for line in bytes_recv_multi_lines if len(line)]
            if not len(lines):
                return

            # discard end_of_msg
            if self.end_of_msg:
                lines = [line for line in lines if line != self.end_of_msg]

            # keep line feed
            if self.keep_line_feed:
                lines = [line + self.receive_lf for line in lines]

            self.logger.info(f'recv_data: {lines}')
            self.queue.extend_items(lines)

        except UnicodeDecodeError as e:
            self.logger.debug(f'Decode error: {e}', ' Resource Bytes:', repr(bytes_recv_line))
        pass

    def new_value(self, value_index=-1):
        """

        :param value_index: get value from queue, default: -1, get last value
        :return:
        """
        if self.queue.__len__() == 0:
            return None
        return self.queue.get_new_value(value_index)


class DequeWithMaxLen(deque):
    """
    FIFO queue with a maximum length. Default is 500

    If the queue is full, the oldest item will be removed.

    put: Put a new value in the queue
    popleft: Remove and return the leftmost item
    get: Alias of popleft
    get_new_value: Get Queue right value, but not remove it
    clear(): Clear the queue
    copy(): Return a shallow copy of the queue
    count(x): Return the number of items in the queue
    extend(iterable): Extend the right side of the queue by appending elements from the iterable
    """
    def __init__(
            self, max_length=500
    ):
        self.max_length = max_length
        super().__init__(maxlen=self.max_length)

    def put(self, item):
        if len(self) == self.max_length:
            self.popleft()
        self.append(item)

    def extend_items(self, iterable):
        new_len = len(self) + len(iterable)
        if new_len >= self.max_length:
            for _ in range(new_len - self.max_length):
                self.popleft()
        self.extend(iterable)

    def popleft(self):
        """
        Remove and return the leftmost item.
        :return:
        """
        return super().popleft()
    
    def get(self):
        return self.popleft()

    def get_new_value(self, value_index=-1):
        """
        Get Queue right value, but not remove it
        :return:
        """
        if len(self) == 0:
            return None
        return self[value_index]



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@File    ：testbase.py
@Author  ：KING
@Date    ：2024/6/11 11:01 
"""
import time
import unittest
from hard_connect import HardConnect


class TestSocket(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        self.hard = HardConnect(conn_type='socket', ip='192.168.0.100', port=5000)
        self.hard.start()
        super().__init__(*args, **kwargs)

    def test_send(self):
        self.hard.send('>vcm force(10)')
        time.sleep(1)
        self.assertEqual(self.hard.queue.__len__(), 1)
        self.hard.queue.clear()

    def test_receive(self):
        print(self.hard.queue)
        self.hard.send('>vcm force(10)')
        time.sleep(1)
        self.assertEqual(self.hard.queue.__len__(), 1)

    def __del__(self):
        self.hard.send('>vcm force(0)')


if __name__ == '__main__':
    unittest.main()





DEFAULT_LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
    },
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(process)d %(thread)d %(module)s %(message)s',
            'datefmt': '%Y-%m-%d %H:%M:%S.%f',
        },
        'simple': {
            'format': '%(asctime)s-%(levelname)s-%(message)s ',
            'datefmt': '%Y-%m-%d %H:%M:%S.%f',
        }
    },
    'handlers': {
        'console': {
            'level': 'INFO',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
        'file': {
            'level': 'DEBUG',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': 'hard_conn.log',  # log file path
            'maxBytes': 1024 * 1024 * 50,  # 50 MB
            'backupCount': 5,
            'formatter': 'verbose',
        }
    },
    'loggers': {
        'hard_conn': {
            'handlers': ['console', 'file'],
            'level': 'INFO',
            'propagate': False,
        },
    }
}






#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@File    ：test.py
@Author  ：KING
@Date    ：2024/6/13 12:51 
"""
import time
import logging
from functools import wraps
from hard_connect.hard_conn import HardConnect


def coroutine(func):
    @wraps(func)
    def primer(*args, **kwargs):
        gen = func(*args, **kwargs)
        next(gen)
        return gen
    return primer


def receive_generator():
    while True:
        data = yield
        print('Data:', data)


def socket_send_eg():
    """
    Example of sending data using socket
    :return:
    """
    hard = HardConnect(conn_type='socket', ip='127.0.0.1', port=60000)
    hard.send('>vcm upload(on)')


def socket_send_receive_eg():
    """
    Example of sending and receiving data using socket
    :return:
    """
    hard = HardConnect(conn_type='socket', ip='127.0.0.1', port=60000)
    receive_data = hard.send_receive('>vcm upload(on)')
    print(receive_data)


def socket_receive_data_with_threading():
    """
    Example of receiving data using socket with threading

    Receive data and put it in a fifo queue
    :return:
    """
    hard = HardConnect(conn_type='socket', ip='127.0.0.1', port=60000)
    hard.start()


def socket_receive_data_with_coroutine():
    """
    Example of receiving data using socket with coroutine

    Receive data and send to a generator
    :return:
    """
    recv = receive_generator()
    next(recv)
    hard = HardConnect(conn_type='socket', ip='127.0.0.1', port=60000, receive_generator=recv)


def socket_connect_set_log_level_and_log_filename():
    """
    Log config is log.py file

    Default log level is logging.INFO
    logging.INFO: use StreamHandler, log to console
    logging.DEBUG: use FileHandler, log to file and console, file name is hard_connect.log in the current directory
    :return:
    """
    hard = HardConnect(
        conn_type='socket', ip='127.0.0.1', port=60000,
        logging_level=logging.DEBUG,
        # logging_filename='/tmp/hard_conn.log'
    )
    hard.send('>vcm upload(on)')


def modbus_tcp_send_eg():
    """
    Example of sending data using modbus tcp
    :return:
    """
    hard = HardConnect(conn_type='modbus_tcp', ip='address', port=502)
    pass

def test_conn():
    hard = HardConnect(
        conn_type='socket', ip='127.0.0.1', port=60000,
        # logging_filename='/tmp/hard_conn.log'
    )

    # hard = HardConnect(conn_type='socket', ip='127.0.0.1', port=60000, logging_level=logging.INFO)
    # hard = HardConnect(conn_type='socket', ip='192.168.0.100', port=5000, logging_level=logging.INFO)
    # hard1 = HardConnect(
    #     conn_type='socket', ip='192.168.0.100', port=5001,  receive_generator=recv_gen
    # )
    # hard = HardConnect(
    #     conn_type='serial', device='/dev/tty.usbmodem1402',
    #     baud_rate=115200, is_read_line=True
    # )
    # hard.start()

    hard = HardConnect(
        conn_type='serial', device='/dev/tty.usbserial-120',
        baud_rate=115200, length=120
    )
    hard.start()
    time.sleep(3)
    hard.disconnect()
    del hard
    # import gc
    # gc.collect()
    # print('----', hard.send_receive('>vcm force(10)'))
    time.sleep(1)
    # print('----', hard.send_receive('>vcm force(10)'))
    # print(hard.send_receive('>vcm force(10)'))
    # hard.send('>vcm force(50)')
    # hard.send_receive('>vcm force(100)')
    # hard.send('>vcm force(0)')
    pass


if __name__ == '__main__':
    test_conn()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@File    ：__init__.py
@Author  ：KING
@Date    ：2024/6/11 18:08 
"""
from hard_connect.hard_conn import HardConnect
from hard_connect.hard_conn import HardConnSock, HardConnSerial


__all__ = ['HardConnect']



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@File    ：hard_conn.py
@Author  ：KING
@Date    ：2024/6/11 11:46 
"""
from threading import Thread
from hard_connect.base_socket import SocketConn
from hard_connect.base_serial import SerialConn
from hard_connect.base_modbus import ModbusTcpClientConn


class HardConnSock(SocketConn, Thread):
    """
    Connect hard device, socket or serial

    Main thread process send, sub thread process receive

    Receive data put in queue, main thread get data from queue

    """
    def __init__(
            self,
            ip=None,            # ip address, if conn_type is socket, ip is necessary
            port=None,          # port, if conn_type is socket, port is necessary
            daemon=True,
            **kwargs
    ):
        self.ip = ip
        self.port = port
        self.kwargs = kwargs
        super().__init__(self.ip, self.port, **self.kwargs)
        self.daemon = daemon

    def run(self):
        self.receive()


class HardConnSerial(SerialConn, Thread):
    """
    Connect hard device, socket or serial

    Main thread process send, sub thread process receive

    Receive data put in queue, main thread get data from queue

    """
    def __init__(
            self,
            device=None,        # device, if conn_type is serial, device is necessary
            baud_rate=None,     # baud rate, if conn_type is serial, baud rate is necessary
            timeout=1,          # timeout, default: 1s, if conn_type is serial, timeout is available
            daemon=True,
            **kwargs
    ):
        self.device = device
        self.baud_rate = baud_rate
        self.timeout = timeout
        self.kwargs = kwargs
        super().__init__(self.device, self.baud_rate, self.timeout, **self.kwargs)
        self.daemon = daemon

    def run(self):
        self.receive()


class HardConnect:

    def __new__(
        cls,
        conn_type,
        ip=None,
        port=None,
        device=None,
        baud_rate=115200,
        **kwargs
    ):
        """
        Connect hard with socket, serial, modbus_tcp.
        Use the same package to communicate.
        Developers do not need to worry about connections and disconnections， etc.
        Save row data to queue.

        Optional arguments (Other parameters/kwargs):
            keep_line_feed: Write queue data to preserve line breaks  default: False
            send_lf:        end data with additional terminator. Default \r\n
            receive_lf:     receive data line separator, Default \r\n
            timeout:        timeout, default: 1s,
            is_read_line:   Serial read row, default: False. call readline() function
            logging_filename: Logging file name, default: None, if set, write log to file. If not set use default log setting
            logging_level:  Logging level, default: logging.INFO, DEBUG: write file and console, INFo: write console
            receive_generator: Generator for receiving data, default: None, if set, use this generator to receive data
            end_of_msg='🔚',  # special tag data, Set the tag end_of_msg, which is not stored in the queue. Default  🔚

        Functions:
            send:
            receive:
            send_receive:
            ...

        :param conn_type:     # hard connect is socket or serial, Only supports socket serial
        :param ip:            # ip address, if conn_type is socket, ip is necessary
        :param port:          # port, if conn_type is socket, port is necessary
        :param device:        # device, if conn_type is serial, device is necessary
        :param baud_rate:     # baud rate, if conn_type is serial, baud rate is necessary. Default: 115200

        :param end_of_msg     # special tag data, Set the tag end_of_msg, which is not stored in the queue. Default  🔚
                                IF end_of_msg set None or '', No processing of data
                                The server returns the end of the data after sending the command.
                                It has no special meaning and is not stored in the queue.
        :param kwargs:        # Other parameters
        :return:              # HardConnSock or HardConnSerial instance
        """

        assert conn_type in ['socket', 'serial', 'modbus_tcp'], 'conn_type must be socket, serial or modbus_tcp '
        if conn_type == 'socket':
            return HardConnSock(ip, port, **kwargs)
        elif conn_type == 'serial':
            return HardConnSerial(device, baud_rate, **kwargs)
        elif conn_type == 'modbus_tcp':
            return ModbusTcpClientConn(ip, port, **kwargs)
        else:
            raise ValueError('conn_type must be socket, serial or modbus_tcp')



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@File    ：base_socket.py
@Author  ：KING
@Date    ：2024/6/11 09:48 
"""
import socket
from typing import Union
from hard_connect.utils import BaseConn
from abc import abstractmethod


class BaseSocket(BaseConn):
    """
    Base socket class, connect and destroy socket.
    init socket and connect, destroy socket and disconnect
    implement send_cmd and recv_msg, Subclass must implement these two methods.

    Don't set timeout, if you set timeout, you will set block mode to blocking mode. Keep socket the default value.
    """
    def __init__(
            self,
            ip, port,
            **kwargs
    ):
        """
        :param ip:   socket connect ip
        :param port: socket connect port
        """
        super().__init__(**kwargs)
        self.ip = ip
        self.port = port if isinstance(port, int) else int(port)
        self.conn = None
        self.connect()

    def connect(self):
        try:
            self.conn: socket.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.timeout and self.conn.settimeout(self.timeout)
            self.conn.connect((self.ip, self.port))
        except Exception as e:
            raise e

    def disconnect(self):
        self.conn and self.conn.close()
        self.conn = None

    @abstractmethod
    def send(self, send_str: Union[str, bytes]):
        """
        send message to socket server,
        message: message + line break
        :param send_str:
        :return: None
        """
        pass

    @abstractmethod
    def receive(self, length: int = None):
        """
        receive message from socket server, Default length is 1024
        :param length:
        :return:
        """
        pass


class SocketConn(BaseSocket):
    """
    Implementing send and receive methods


    """
    def __init__(
            self,
            ip: str,
            port: int,
            length: int = 1024,
            **kwargs
    ):
        """

        :param ip:       server ip
        :param port:     server port
        :param length:   receive data length
        :param kwargs:
        """
        self.length = length
        super().__init__(ip, port, **kwargs)

    def send(self, send_str: Union[str, bytes]):
        """
        send message to socket server.

        :param send_str: message
        :return:
        """
        if not self.conn:
            raise Exception(f"socket {self.ip}:{self.port} is not connected")

        try:
            if isinstance(send_str, bytes):
                self.conn.sendall(send_str)
                self.logger.info(f'send bytes length: {len(send_str)}')
            else:
                send_msg = send_str + self.send_lf
                self.logger.info(f'send: {send_msg}')
                self.conn.sendall(send_msg.encode())
        except Exception as e:
            self.logger.error(f"send socket {self.ip}:{self.port} error: {e}")

    def receive(
            self,
            length: int = None,
            receive_data: bytes = None,
    ) -> Union[bytes, None]:
        """
        receive message from socket server, Default length is 1024

        Process multi-line data, Process coding is bytes, not str
        The received data may be incomplete or line by line, so we need to process it line by line

        Loop receive data,
        Return the received data when receive_data is not None or timeout is reached

        If set length
        When the received data is greater than or equal to length, return the received data
        :param length:        single receive data length, if length is None, use self.length
        :param receive_data:
        :param :
        :return:
        """
        bytes_line_break = self.receive_lf.encode()
        bytes_end_of_msg = self.end_of_msg.encode()

        socket_data = b''
        try:
            while self.conn:
                bytes_recv_data = self.conn.recv(self.length if length is None else length)
                socket_data += bytes_recv_data

                self.receive_generator and self.receive_generator.send(bytes_recv_data)
                # send command and waiting for server response data
                if receive_data is not None:
                    receive_data += bytes_recv_data

                # Data write queue by line
                if bytes_line_break in socket_data:

                    # delete empty line
                    lines = [_line for _line in socket_data.split(bytes_line_break) if len(_line)]
                    if len(lines) == 0:
                        socket_data = b''
                    elif len(lines) == 1 or socket_data.endswith(bytes_line_break):
                        self.put_queue(bytes_recv_multi_lines=lines)
                        socket_data = b''
                    else:
                        socket_data = lines[-1]
                        self.put_queue(bytes_recv_multi_lines=lines[:-1])

                # when receive_data is not None. First data put queue then return data, end receive of loop
                if ((receive_data and bytes_end_of_msg in receive_data)
                        or (length and len(receive_data) >= length)):
                    return receive_data
        except socket.timeout:
            return receive_data if receive_data is not None else socket_data
        except IOError as e:
            self.logger.error('Socket Exception:', e)

    def send_receive(self, send_str: Union[str, bytes], receive_length=None) -> str:
        """
        send message to socket server and receive message from socket server

        :param send_str:
        :param receive_length:
        :return:
        """
        self.send(send_str)
        receive_data = b''
        receive_data = self.receive(
            length=receive_length,
            receive_data=receive_data
        )
        return receive_data.decode().strip() if receive_data else None



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@File    ：base_serial.py
@Author  ：KING
@Date    ：2024/6/11 09:48 
"""
import time
import serial
from typing import Union
from abc import abstractmethod
from hard_connect.utils import BaseConn


class BaseSerial(BaseConn):
    def __init__(
            self,
            device,
            baud_rate=115200,
            timeout=1,
            **kwargs
    ):
        """
        Serial communication base class.
        init serial and connect, destroy serial and disconnect
        Subclass must implement these two methods.

        :param device:      serial device, example: /dev/ttyUSB0;
        :param baud_rate:   serial baud rate, default: 115200;
        :param timeout:     timeout, default: 1s;
        """
        super().__init__(**kwargs)
        self.device = device
        self.baud_rate = baud_rate
        self.timeout = timeout
        self.kwargs = kwargs
        self.conn = None
        self.connect()
        self.alive = None

    def connect(self):
        """
        if serial need to set some parameters, you can set it in the kwargs
        :return:
        """
        try:
            self.conn = serial.Serial(
                self.device, self.baud_rate, timeout=self.timeout,
            )
        except Exception as e:
            raise e

    def disconnect(self):
        if self.conn and self.conn.isOpen():
            self.conn.flushOutput()
            self.conn.close()
            self.conn = None

    @abstractmethod
    def send(self, send_str: str):
        pass

    @abstractmethod
    def receive(self, length: int = None):
        pass


class SerialConn(BaseSerial):
    """
    Implementing send and receive methods


    """

    def __init__(
            self,
            device: str,
            baud_rate: int,
            timeout: int = 1,
            is_read_line: bool = False,
            length: int = 1024,
            **kwargs
    ):
        """
        Default read data is readline, if you want to read a fixed length of data, you can set is_read_line to False
        If is_read_line is True, Hardware return data must have a line break, otherwise it will block.

        :param device:       serial device, example: /dev/ttyUSB0;
        :param baud_rate:    serial baud rate, default: 115200;
        :param timeout:      serial timeout, default: 1s;
        :param is_read_line: read row, default: False;
        :param kwargs:
        """
        self.is_read_line = is_read_line
        self.length = length
        super().__init__(device, baud_rate, timeout, **kwargs)

    @property
    def is_open(self):
        return self.conn and self.conn.isOpen()

    def send(self, send_str: str):
        if not self.conn or not self.conn.isOpen():
            raise Exception(f"Serial port {self.device} is not connection or open")

        try:
            self.conn.flushInput()  # Clear the input buffer of the serial port
            self.conn.flushOutput()  # clear the output buffer of the serial port

            send_msg = send_str + self.send_lf
            self.logger.info(f'send: {send_msg}')
            self.conn.write(send_msg.encode())
            self.conn.flush()

        except Exception as e:
            self.logger.error(f"send serial {self.device} error: {e}")
    
    def receive(
            self,
            length: int = None,
            receive_data: bytes = None,
    ) -> Union[bytes, None]:
        """
        receive message from serial port, Default length is 1024

        :param length:        If is_read_line is True, length and self.length is invalid.
                              If is_read_line is False, when length is not None, read length data. otherwise  read self.length data  # noqa
        :param receive_data:  If receive_data is not None, append data to receive_data. Use send_receive method
        :return:
        """
        bytes_end_of_msg = self.end_of_msg.encode()
        start_time = time.time()

        while self.conn.is_open:
            try:
                if self.is_read_line:
                    _bytes = self.conn.readline()
                else:
                    _bytes = self.conn.read(self.length if length is None else length)
            except serial.SerialException as e:
                self.logger.error(f"Serial {self.device} error: {e}  serial open state: {self.conn.is_open}")
                break

            self.receive_generator and self.receive_generator.send(_bytes)
            len(_bytes) and self.put_queue(_bytes)

            # send command and waiting for server response data
            if receive_data is None:
                continue

            # receive single data
            if len(_bytes):
                receive_data += _bytes

            if (bytes_end_of_msg in receive_data
                    or time.time() - start_time > self.timeout):  # noqa
                return receive_data

    def send_receive(self, send_str: Union[str, bytes], receive_length: int = None) -> str:
        """
        send message to socket server and receive message from socket server

        :param send_str:
        :param receive_length:
        :return:
        """
        self.send(send_str)
        receive_data = b''

        receive_data = self.receive(
            length=receive_length,
            receive_data=receive_data
        )
        return receive_data.decode().strip() if receive_data else None



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@File    ：test_vcm.py
@Author  ：KING
@Date    ：2024/6/17 16:16 
"""
import re
import time
from hard_connect import HardConnect


def get_force(socket5000: HardConnect):
    getdata = socket5000.send_receive(">vcm getforce()")
    if getdata.startswith('<vcm getforce success.'):
        pattern = r'\[([0-9.]+)\s*[a-zA-Z]*\]'
        match = re.search(pattern, getdata)
        if match:
           return match.group(1)
    return None


if __name__ == '__main__':
    vcm_socket5000 = HardConnect(
        conn_type='socket', ip="192.168.0.100", port=5000
    )
    with HardConnect(
        conn_type='socket', ip="192.168.0.100", port=5001, console_print=False,
    ) as vcm_socket5001:
        vcm_socket5001.start()
        vcm_socket5000.send_receive('>vcm upload(on)')

        time.sleep(60)
        exit(1)

    cheng_serial = HardConnect(
        conn_type='serial',
        device='/dev/tty.usbserial-140',
        baud_rate=115200,
        timeout=0.2,
        is_read_line=False,
        length=120,
        console_print=False,
    )
    cheng_serial.start()

    vcm_socket5000.send(">loadcell reset()")
    time.sleep(3)
    text = ['0.20', '0.25', '0.30', '0.35', '0.40']
    pre_vcm_val, pre_cheng_val = 0, 0
    for volt in text:
        vcm_socket5000.send(f'>vcm current({volt})')
        time.sleep(7)

        force = get_force(vcm_socket5000)
        val = cheng_serial.new_value()
        print('force: ', force, '  val: ', val)


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@File    ：setup.py
@Author  ：KING
@Date    ：2024/6/11 18:22 
"""
from setuptools import setup, find_packages

setup(
    name='hard_connect',
    version='0.3.1',
    packages=find_packages(),
    install_requires=[
        # 依赖列表
        'pyserial==3.5',
        'pymodbus==3.6.8',
    ],
    author='kingduyin',
    author_email='kingduyin@gmail.com',
    description='Connect hard device, socket or serial',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    # url='https://github.com/yourusername/mymodule',  # 模块的主页
    classifiers=[
        # 模块分类
    ],
    python_requires='>=3.6',
)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@File    ：base_modbus.py
@Author  ：KING
@Date    ：2024/6/20 09:11 
"""
from pymodbus.client import ModbusTcpClient
from typing import Union
from hard_connect.utils import BaseConn


class ModbusTcpClientConn(BaseConn):
    """
    """
    def __init__(
            self,
            ip: str,
            port: int,
            timeout: int = 1,
            **kwargs
    ):
        """

        :param ip:       server ip
        :param port:     server port
        :param length:   receive data length
        :param kwargs:
        """
        super().__init__(ip, port, **kwargs)
        self.ip = ip
        self.port = port if isinstance(port, int) else int(port)
        self.timeout = timeout
        self.conn = None
        self.connect()

    def connect(self):
        try:
            self.conn: ModbusTcpClient = ModbusTcpClient(self.ip, self.port, timeout=self.timeout)
        except Exception as e:
            raise e

    def disconnect(self):
        self.conn and self.conn.close()
        self.conn = None

    def send(self, send_str: Union[str, bytes]):
        """
        :param send_str: message
        :return:
        """
        pass

    def receive(
            self,
            length: int = None,
            receive_data: bytes = None,
    ) -> Union[bytes, None]:
        """
        :param length:
        :param receive_data:
        :param :
        :return:
        """
        pass

    def send_receive(self, send_str: Union[str, bytes], receive_length=None) -> str:
        """
        :param send_str:
        :param receive_length:
        :return:
        """
        self.send(send_str)
        receive_data = b''
        receive_data = self.receive(
            length=receive_length,
            receive_data=receive_data
        )
        return receive_data.decode().strip() if receive_data else None
